/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "COPYING_Apache"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */


#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/mman.h>
#include <errno.h>

#include "../kernel_trace_hmtt3/kernel_trace_hmtt3.h"

static char *kernel_trace_path = "/dev/kernel_trace_hmtt3";
static char *hmtt_control_path = "/dev/config_space";

int is_pt_addr_trace = 0;

#define CFG_SIZE        0x1000U     //this size is used for config_space mmap space size

void test_hmtt_trace(int fd)
{
    int command;
    int wr_cnt;

#if 1
    command = CMD_IOREMAP_WRITE_TEST;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Begin HMTT Trace: write 0 failure!\n");
    else
        printf("Begin HMTT Trace:  write 0 success!\n");
#endif
}


void begin_hmtt_trace(int fd)
{
    int command;
    int wr_cnt;

#if 1
    command = CMD_START_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Begin HMTT Trace: write 0 failure!\n");
    else
        printf("Begin HMTT Trace:  write 0 success!\n");
#endif
}


void begin_page_table_trace_test(int fd)
{
    int command;
    int wr_cnt;

#if 1
	command = CMD_BEGIN_KERNEL_TRACE;
	wr_cnt = write(fd, &command, sizeof(int));
	if( wr_cnt != sizeof(int) )
	{
		printf("#### Begin Kernel Trace: write 0 failure!\n");
	}
	else
	{
		printf("$$$$ Being Kernel Trace: write 0 success!\n");
	}

//////////////////////////////// Begin dump page table address trace  ////////////////////////////////
	if(is_pt_addr_trace)
	{
		command = CMD_PT_ADDR_START_TRACE;
		wr_cnt = write(fd, &command, sizeof(int));
        	if( wr_cnt != sizeof(int) )
        	{
                	printf("#### Begin PT Addr Trace: write 0 failure!\n");
        	}
        	else
        	{
                	printf("$$$$ Being PT Addr Trace: write 0 success!\n");
        	}
	}
////////////////////////////  Begin Dump Page Table Trace   //////////////////////////////////////
    command = CMD_DUMP_PAGE_TABLE_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Begin Dump Page Table Trace: write 0 failure!\n");
    else
        printf("Begin Dump Page Table Trace: write 0 success!\n");

    ///////////////////////  Begin dump page table entry (pte) trace    ///////////////////////////////////
    command = CMD_PAGE_TABLE_START_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Begin Page Table Trace: write 0 failure!\n");
    else
        printf("Begin Page Table Trace: write 0 success!\n");

	/*******************  Begin dump context switch  ******************************/
	command = CMD_CONTEXT_SWITCH_START_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Begin Context Switch Trace: write 0 failure!\n");
    else
        printf("Begin Context Switch Trace: write 0 success!\n");
#endif
}

void begin_pid_comm_trace_test(int fd)
{
    int command;
    int wr_cnt;

#if 1
    
    command = CMD_PID_COMM_START_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Begin PID & Command Trace: write 0 failure!\n");
    else
        printf("Begin PID & Command Table Trace: write 0 success!\n");
#endif
}

void begin_context_switch_trace_test(int fd)
{
    int command;
    int wr_cnt;

#if 1
    
    command = CMD_CONTEXT_SWITCH_START_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Begin Context Switch Trace: write 0 failure!\n");
    else
        printf("Begin Context Switch Trace: write 0 success!\n");
#endif
}


void begin_dump_page_table_trace_test(int fd)
{
    int command;
    int wr_cnt;

#if 1

    command = CMD_DUMP_PAGE_TABLE_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Begin Dump Page Table Trace: write 0 failure!\n");
    else
        printf("Begin Dump Page Table Trace: write 0 success!\n");

#endif
}


void begin_sata_dma_trace_test(int fd)
{
    int command;
    int wr_cnt;

#if 1
    command = CMD_SATA_START_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Begin SATA Trace: write 0 failure!\n");
    else
        printf("Begin SATA Trace: write 0 success!\n");
#endif
}


void begin_e1000_dma_trace_test(int fd,int start_ide)
{
    
    int command;
    int wr_cnt;

    printf("we are in e1000 dma_trace_test!\n");

    if(start_ide == 1)
    {
        //start ide trace
        command = CMD_SATA_START_TRACE;
        wr_cnt = write(fd,&command,sizeof(int));

        if(wr_cnt != sizeof(int))
            printf("Begin SATA Trace: write 0 failure!\n");
        else
            printf("Begin SATA Trace: write 0 success!\n");
    }
    //start e1k TX trace
    command = CMD_E1K_START_TX_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Begin E1K_TX Trace: write 0 failure!\n");
    else
        printf("Begin E1K_TX Trace: write 0 success!\n");

    //start e1k RX trace
    command = CMD_E1K_START_RX_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Begin E1K_RX Trace: write 0 failure!\n");
    else
        printf("Begin E1K_RX Trace: write 0 success!\n");
}

void begin_all_trace_test(int fd)
{
    
    int command;
    int wr_cnt;

    printf("we are in all _trace_test!\n");

    command = CMD_DUMP_PAGE_TABLE_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Begin Dump Page Table Trace: write 0 failure!\n");
    else
        printf("Begin Dump Page Table Trace: write 0 success!\n");


    //dump context switch
    command = CMD_CONTEXT_SWITCH_START_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Begin Context Switch Trace: write 0 failure!\n");
    else
        printf("Begin Context Switch Trace: write 0 success!\n");

    //start page_table
    command = CMD_PAGE_TABLE_START_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Begin Page Table Trace: write 0 failure!\n");
    else
        printf("Begin Page Table Trace: write 0 success!\n");

    //start sata trace
    command = CMD_SATA_START_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Begin SATA Trace: write 0 failure!\n");
    else
        printf("Begin SATA Trace: write 0 success!\n");
    
    //start e1k TX trace
    command = CMD_E1K_START_TX_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Begin E1K_TX Trace: write 0 failure!\n");
    else
        printf("Begin E1K_TX Trace: write 0 success!\n");

    //start e1k RX trace
    command = CMD_E1K_START_RX_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Begin E1K_RX Trace: write 0 failure!\n");
    else
        printf("Begin E1K_RX Trace: write 0 success!\n");

    //pid_comm
    command = CMD_PID_COMM_START_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Begin PID & Command Trace: write 0 failure!\n");
    else
        printf("Begin PID & Command Table Trace: write 0 success!\n");
 
}

#define PAGE_SIZE (1024*1024*1024)
main(int argc,char ** argv)
{
    int fd;
    int cmd;
    char buf[4];
    char *ioremap_buf;
    int i,j,k;
    char ch;
    char* buffer;

	/*************************************************************************************************/
	int hmtt_control_fd;
	void* config_space_base = NULL;
	/************************************************************************************************/

    if(argc != 2 && argc != 3)
    {
        printf("Usage: ./begin_hmtt_trace cmd [pt_addr_trace]\n");
        printf("\n");
        printf("cmd           = 0  HMTT Memory Trace\n");
        printf("                1  HMTT Trace & Kernel Trace(Page Table & Context Switch Trace)\n");
	printf("                2  Only Kernel Trace (pte & cs Trace)\n");
	printf("pt_addr_trace = 1 Dump Page Table Address Trace\n");
	printf("              = 0 No PT_ADDR Trace\n");
        //printf("      2  SATA & E1K DMA trace\n");
        //printf("      3  Page Table trace & context switch\n");
        //printf("      4  Dump Page Table trace\n");
        //printf("      5  Dump pid & command\n");
        //printf("      6  Dump context switch\n");
	//printf("$$$$ Now we just support 3 for dump page tabel trace and context switch\n");
        //printf("     99  All traces\n");
        //printf("    100  HMTT traces\n");
        //printf("   1000  Test HMTT traces\n");
        printf("\n");
        
        return 0;
    }

    fd = open(kernel_trace_path,O_RDWR);
    if(fd < 0)
    {
        printf("#### can not open %s, errno = %d!\n", kernel_trace_path,errno);
        return -8;
    }
	printf("$$$$ Open dev %s successfully\n", kernel_trace_path); 

	read(fd,buf,4);
    	cmd = atoi(argv[1]);
  	if( cmd != 0 && cmd != 1 && cmd != 2){
		printf("$$$$ Now we just support cmd 0, 1 or 2 for HMTT, dump page tabel and context switch Trace\n");
		return -8;	
	}
	if(argc == 3)
	{
		is_pt_addr_trace = atoi(argv[2]);
	}
/**********************************************************
** Begin HMTT3 trace, 
** Just ignore it
**********************************************************/
	if( cmd != 2 )
	{
	// we want to start HMTT Trace
		hmtt_control_fd = open(hmtt_control_path, O_RDWR);
		if( hmtt_control_fd < 0 )
		{
			printf("#### Failed to open %s, errno = %d\n", hmtt_control_path, errno);
			return -8;
		}
		printf("$$$$ Open dev %s successfully\n", hmtt_control_path);
		config_space_base = mmap( 0,CFG_SIZE,PROT_READ|PROT_WRITE,MAP_SHARED,hmtt_control_fd,0 );
	    	if( config_space_base < 0 ){
        		printf("#### Failed to mmap %s into process space, errno=%d\n", hmtt_control_path, errno);
        		return -1;
	    	}
        	printf("$$$$ mmap %s into process addr space successfully\n", hmtt_control_path);
	
		//send a tag to hmtt for start trace
		ch = *(char *)(config_space_base + 64*CMD_START_TRACE);
		ch = *(char *)(config_space_base + 64*CMD_START_TRACE);
		ch = *(char *)(config_space_base + 64*CMD_START_TRACE);
               	ch = *(char *)(config_space_base + 64*CMD_START_TRACE);
                ch = *(char *)(config_space_base + 64*CMD_START_TRACE);
                ch = *(char *)(config_space_base + 64*CMD_START_TRACE);
	}
/*******************************************************
** End of begin HMTT3 trace
********************************************************/
   
#if 1


    //start trace
    if(cmd == 0)//hmtt trace
	//do nothing here, we have just start hmtt trace above
	;
    else if(cmd == 1)//hmtt & kernel trace
	begin_page_table_trace_test(fd);
    
    else if(cmd == 2)//kernel trace
        begin_page_table_trace_test(fd);
    //else if(cmd == 3)//ide and e1k
    //    begin_page_table_trace_test(fd);
    //else if(cmd == 4)//ide and e1k
    //    begin_dump_page_table_trace_test(fd);
    //else if(cmd == 5)//ide and e1k
    //    begin_pid_comm_trace_test(fd);
    //else if(cmd == 6)//ide and e1k
    //    begin_context_switch_trace_test(fd);
    /*else if(cmd == 99)//ide and e1k
        begin_all_trace_test(fd);
    else if(cmd == 100)//hmtt
        begin_hmtt_trace(fd);
    else if(cmd == 1000)//hmtt
        test_hmtt_trace(fd);*/
    else
       printf("#### param error!\n");

#endif
	if( cmd != 2 )
	{
		munmap( config_space_base, CFG_SIZE );
    		close(hmtt_control_fd);
	}

    	close(fd);

        printf("$$$$ Exit from begin_hmtt_trace\n");

    	return 0;
}
