/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "COPYING_Apache"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */


#include <asm/unistd.h> 
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/ioctl.h>
#include <string.h>
#include <sched.h>
#include <signal.h>
#include <dirent.h>
#include <errno.h>

#include "../kernel_trace_hmtt3/kernel_trace_hmtt3.h"

static char *kernel_trace_path = "/dev/kernel_trace_hmtt3";

extern int errno;

//global variable
int ff;
unsigned long dev_size, buffer_size;  
unsigned char *p_kernel_trace = NULL, *p_kernel_trace_buf = NULL;
char filename[64];
FILE *fp, *fpid, *pp;
char  cmd[64];
char  one_line[256];
int   global_max_pid; 
int   kernel_trace_offset;
int   trace_size;

unsigned long *dev_writeptr;
unsigned long *dev_readptr;

int dump_processes_pid();
/*
 * hanlde the Ctrl-C signal while kill the process
 */
void ctrl_c_op(int signo)
{
    int command, wr_cnt;
	
	command = CMD_WRITE_KERNEL_TRACE_END_TAG;
	wr_cnt = write(ff,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Write Kernel Trace End Tag Error\n");
    else
        printf("Write Kernel Trace End Tag OK\n");

    printf("[Kernel_Trace]  Entering Ctrl-C Handler\n");
    
    //printf("readptr = %x, writeptr = %x\n", *dev_readptr - kernel_trace_offset, *dev_writeptr-kernel_trace_offset);

    if(*dev_readptr == kernel_trace_offset){
        fwrite(p_kernel_trace_buf + *dev_readptr, *dev_writeptr - kernel_trace_offset, 1, fp);
    } else if(*dev_readptr == kernel_trace_offset + buffer_size/2){
        if(*dev_writeptr < kernel_trace_offset + buffer_size/2){
            fwrite(p_kernel_trace_buf + *dev_readptr, buffer_size/2, 1, fp);
            fwrite(p_kernel_trace_buf + kernel_trace_offset, *dev_writeptr - kernel_trace_offset, 1, fp);
        } else {
            fwrite(p_kernel_trace_buf + *dev_readptr, *dev_writeptr - *dev_readptr, 1, fp);
        }
    } else {
        printf("Write File Error\n");
    }

    printf("readptr = %x, writeptr = %x\n", *dev_readptr - kernel_trace_offset, *dev_writeptr-kernel_trace_offset);

    command = CMD_RESET_BUFFER;
    wr_cnt = write(ff,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Reset Buffer Error\n");
    else
        printf("Reset Buffer OK\n");

    munmap(p_kernel_trace,dev_size);
    close(ff);

    fclose(fp);
    printf("[Kernel_Trace]  Collect Kernel Trace OK.\n");

    exit(0);
}
	
int main(int argc, char **argv,char **env)
{
    int i;
    int m=0;
    int scount=0;
    unsigned int *pidbuf_num;
    char         *pidbuf;
    int          pid;
    int          sleep_time;
    int          trace_bw;
    unsigned long          last_writeptr;
    unsigned long          write_sum = 0;
   
     if(argc != 3)
     {
         printf("Usage:./ page_table_dump filename time_interval\n");
         return 0;
     }
     memset(filename, 0, 64);
     strcpy(filename, argv[1]);
     strcat(filename, ".kt");
     if((fp = fopen(filename, "w+")) == NULL){
          printf("Open file %s error\n", filename);
          return 0;
     }
 
     sleep_time = atoi(argv[2]);
     if(sleep_time == 0)
         sleep_time = 80;

    /* 
     * Step.0 register sigaction to reponse Ctrl+C
     */
    struct sigaction s;

    s.sa_handler = (void*)ctrl_c_op;
    sigemptyset(&s.sa_mask);
    s.sa_flags = 0;
    if(sigaction(SIGINT, &s, (struct sigaction*)NULL)){
        printf("register sigaction error\n");
        exit(0);
    }
    
    /*
     *  Step.1 Init pro device and malloc buffers to store traces
     */	

    ff = open(kernel_trace_path, O_RDWR);
    //printf("open %s in %d\n", kernel_trace_path, ff);
    if( ff < 0 ){
        printf("#### open %s failed\n",kernel_trace_path);
        exit(-8);
    }

    dev_size = KERNEL_TRACE_SIZE << 20; //16384*4096;//ioctl(ff,PRO_IOCQ_PAGENUM) * 4096;	  //get pro's buffer size

   
    p_kernel_trace = (unsigned char*)mmap(0, dev_size, PROT_READ|PROT_WRITE, MAP_SHARED,ff,0);
    if ((long long)p_kernel_trace < 0){
	printf("#### error in mmap\n");
        switch(errno){
            case EACCES: printf("EACCES\n"); break;
            case EAGAIN: printf("EAGAIN\n"); break;
            case EBADF: printf("EBADF\n"); break;
            case EINVAL: printf("EINVAL\n"); break;
            case ENFILE: printf("ENFILE\n"); break;
            case ENODEV: printf("ENODEV\n"); break;
            case ENOMEM: printf("ENOMEM\n"); break;
            case EPERM: printf("EPERM\n"); break;
            //case ETXTBSY: printf("ETXTBSY\n"); break;
            //case SIGSEGV: printf("SIGSEGV\n"); break;
            //case SIGBUS: printf("SIGBUS\n"); break;
        }

        return -1;
    }

    // dump existing page table of system
    buffer_size = KERNEL_TRACE_BUF_SIZE << 20; //16384*4096;//ioctl(ff,PRO_IOCQ_PAGENUM) * 4096;	  //get pro's buffer size
    kernel_trace_offset = 0; //KERNEL_TRACE_WRRD_PTR_SIZE << 20;

    p_kernel_trace_buf = p_kernel_trace + (KERNEL_TRACE_WRRD_PTR_SIZE << 20);
    /*
     *  Step.2 read trace from pro's buffer
     */	

    write_sum = 0;
    dev_writeptr = (unsigned long*)p_kernel_trace;
    dev_readptr = (unsigned long*)(dev_writeptr + 1);
    last_writeptr = *dev_writeptr;
    while(1){     

        if(last_writeptr <= *dev_writeptr)
             trace_bw = ((*dev_writeptr - last_writeptr) / sleep_time) >> 10 ;
        else trace_bw = ((buffer_size - last_writeptr + *dev_writeptr) / sleep_time) >> 10;
        last_writeptr = *dev_writeptr;

        //printf("readptr = %x, writeptr = %x\n", *dev_readptr - kernel_trace_offset, *dev_writeptr-kernel_trace_offset);
        if(*dev_readptr == kernel_trace_offset){
            if(*dev_writeptr >= kernel_trace_offset + buffer_size/2){
                fwrite(p_kernel_trace_buf + *dev_readptr, buffer_size/2, 1, fp);
                *dev_readptr = kernel_trace_offset + buffer_size/2;
                write_sum += buffer_size/2;
            }
        } else if(*dev_readptr == kernel_trace_offset + buffer_size/2){
            if(*dev_writeptr < kernel_trace_offset + buffer_size/2){
                fwrite(p_kernel_trace_buf + *dev_readptr, buffer_size/2, 1, fp);
                *dev_readptr = kernel_trace_offset;
                write_sum += buffer_size/2;
            }
        } else {
            printf("Write File Error\n");
        }

        printf("readptr = %x, writeptr = %x, bw=%d\n", *dev_readptr - kernel_trace_offset, *dev_writeptr-kernel_trace_offset, trace_bw);
        sleep(sleep_time);

        if(write_sum >= buffer_size && trace_bw <= 512){
            sleep_time = 80;
        }

    }//endof 	while(1)
	return 0;
 }
 
