/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "COPYING_Apache"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */


#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/mman.h>
#include <errno.h>

#include "../kernel_trace_hmtt3/kernel_trace_hmtt3.h"

static char *kernel_trace_path = "/dev/kernel_trace_hmtt3";
static char *hmtt_control_path = "/dev/config_space";

int is_pt_addr_trace = 0;

#define CFG_SIZE        0x1000U     //this size is used for config_space mmap space size


void stop_hmtt_trace(int fd)
{
    int command;
    int wr_cnt;

#if 1
    //stop and clear trace
    command = CMD_STOP_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Stop HMTT Trace: write 2 failure!\n");
    else
        printf("Stop HMTT Trace: write 2 success!\n");
#endif
}

void stop_page_table_trace_test(int fd)
{
    int command;
    int wr_cnt;

#if 1
    //stop and clear trace
    command = CMD_PAGE_TABLE_STOP_AND_CLEAR_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Stop Page Table Trace: write 2 failure!\n");
    else
        printf("Stop Page Table Trace: write 2 success!\n");

//stop the context switch	
	command = CMD_CONTEXT_SWITCH_STOP_AND_CLEAR_TRACE; //CMD_CONTEXT_SWITCH_STOP_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Stop Context Switch Trace: write 2 failure!\n");
    else
        printf("Stop Context Switch Trace: write 2 success!\n");

//////////////////////////////// Stop dump page table address trace  ////////////////////////////////
        if(is_pt_addr_trace)
        {
                command = CMD_PT_ADDR_STOP_TRACE;
                wr_cnt = write(fd, &command, sizeof(int));
                if( wr_cnt != sizeof(int) )
                {
                        printf("#### Stop PT Addr Trace: write 0 failure!\n");
                }
                else
                {
                        printf("$$$$ Stop PT Addr Trace: write 0 success!\n");
                }
        }

    //dump the trace to log file
    command = CMD_END_KERNEL_TRACE; //CMD_END_AND_DUMP_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Dump Trace: write 3 failure!\n");
    else
        printf("Dump Trace: write 3 success!\n");
#endif
}

void stop_pid_comm_trace_test(int fd)
{
    int command;
    int wr_cnt;

#if 1
    //stop and clear trace
    command = CMD_PID_COMM_STOP_AND_CLEAR_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Stop PID & Command Trace: write 2 failure!\n");
    else
        printf("Stop PID & Command Trace: write 2 success!\n");
#endif
}

void stop_context_switch_trace_test(int fd)
{
    int command;
    int wr_cnt;

#if 1
    //stop and clear trace
    command = CMD_CONTEXT_SWITCH_STOP_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Stop Context Switch Trace: write 2 failure!\n");
    else
        printf("Stop Context Switch Trace: write 2 success!\n");
#endif
}


void stop_dump_page_table_trace_test(int fd)
{
    int command;
    int wr_cnt;

#if 1

    //dump the trace to log file
    command = CMD_END_AND_DUMP_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Dump Trace: write 3 failure!\n");
    else
        printf("Dump Trace: write 3 success!\n");
#endif
}


void stop_sata_dma_trace_test(int fd)
{
    int command;
    int wr_cnt;

#if 1
    //stop and clear trace
    command = CMD_SATA_STOP_AND_CLEAR_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Stop SATA Trace: write 2 failure!\n");
    else
        printf("Stop SATA Trace: write 2 success!\n");

    //dump the trace to log file
    command = CMD_END_AND_DUMP_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Dump Trace: write 3 failure!\n");
    else
        printf("Dump Trace: write 3 success!\n");
#endif
}


void stop_e1000_dma_trace_test(int fd,int start_ide)
{
    
    int command;
    int wr_cnt;

    printf("we are in e1000 dma_trace_test!\n");

    //stop and clear TX trace
    command = CMD_E1K_STOP_AND_CLEAR_TX_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Stop E1K_TX Trace: write 2 failure!\n");
    else
        printf("Stop E1K_TX Trace: write 2 success!\n");

    //stop and clear RX trace
    command = CMD_E1K_STOP_AND_CLEAR_RX_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Stop E1K_RX Trace: write 2 failure!\n");
    else
        printf("Stop E1K_RX Trace: write 2 success!\n");

    if(start_ide == 1)
    {
        //stop and clear trace
        command = CMD_SATA_STOP_AND_CLEAR_TRACE;
        wr_cnt = write(fd,&command,sizeof(int));

        if(wr_cnt != sizeof(int))
            printf("Stop SATA Trace: write 2 failure!\n");
        else
            printf("Stop SATA Trace: write 2 success!\n");
    }

    //dump the trace to log file
    command = CMD_END_AND_DUMP_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Dump Trace: write 3 failure!\n");
    else
        printf("Dump Trace: write 3 success!\n");
}

void stop_all_trace_test(int fd)
{
    
    int command;
    int wr_cnt;

    command = CMD_CONTEXT_SWITCH_STOP_AND_CLEAR_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Stop Context Switch Trace: write 2 failure!\n");
    else
        printf("Stop Context Switch Trace: write 2 success!\n");

    //stop and clear pid_comm trace
    command = CMD_PID_COMM_STOP_AND_CLEAR_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Stop PID & Command Trace: write 2 failure!\n");
    else
        printf("Stop PID & Command Trace: write 2 success!\n");

    //stop page table traces
    command = CMD_PAGE_TABLE_STOP_AND_CLEAR_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Stop Page Table Trace: write 2 failure!\n");
    else
        printf("Stop Page Table Trace: write 2 success!\n");

 
    //stop and clear TX trace
    command = CMD_E1K_STOP_AND_CLEAR_TX_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Stop E1K_TX Trace: write 2 failure!\n");
    else
        printf("Stop E1K_TX Trace: write 2 success!\n");

    //stop and clear RX trace
    command = CMD_E1K_STOP_AND_CLEAR_RX_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Stop E1K_RX Trace: write 2 failure!\n");
    else
        printf("Stop E1K_RX Trace: write 2 success!\n");

    //stop and clear trace
    command = CMD_SATA_STOP_AND_CLEAR_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Stop SATA Trace: write 2 failure!\n");
    else
        printf("Stop SATA Trace: write 2 success!\n");

    //dump the trace to log file
    command = CMD_END_AND_DUMP_TRACE;
    wr_cnt = write(fd,&command,sizeof(int));

    if(wr_cnt != sizeof(int))
        printf("Dump Trace: write 3 failure!\n");
    else
        printf("Dump Trace: write 3 success!\n");
}

#define PAGE_SIZE (1024*1024*1024)
main(int argc,char ** argv)
{
    int fd;
    int cmd;
    char buf[4];
    char *ioremap_buf;
    int i,j,k;
    char ch;

	/*************************************************************************************************/
        int hmtt_control_fd;
        void* config_space_base = NULL;
        /************************************************************************************************/

	if(argc != 2 && argc != 3)
    	{
        	printf("Usage: ./stop_hmtt_trace cmd [pt_addr_trace]\n");
        	printf("\n");
       	 	printf("cmd           = 0  HMTT Memory Trace\n");
        	printf("                1  HMTT Trace & Kernel Trace(Page Table & Context Switch Trace)\n");
        	printf("                2  Only Kernel Trace (pte & cs Trace)\n");
		printf("pt_addr_trace = 1 Dump Page Table Address Trace\n");
        	printf("              = 0 No PT_ADDR Trace\n");
        	//printf("      2  SATA & E1K DMA trace\n");
        	//printf("      3  Page Table trace & context switch\n");
        	//printf("      4  Dump Page Table trace\n");
        	//printf("      5  Dump pid & command\n");
        	//printf("      6  Dump context switch\n");
        	//printf("$$$$ Now we just support 3 for dump page tabel trace and context switch\n");
       	 	//printf("     99  All traces\n");
        	//printf("    100  HMTT traces\n");
        	//printf("   1000  Test HMTT traces\n");
        	printf("\n");

        	return 0;
    	}

    fd = open(kernel_trace_path,O_RDWR);
    if(fd < 0)
    {
        printf("#### can not open %s, errno = %d!\n",kernel_trace_path , errno);
        return -8;
    } 
	
  	cmd = atoi(argv[1]); 
	if( cmd != 0 && cmd != 1 && cmd != 2){
                printf("$$$$ Now we just support cmd 0, 1 or 2 for HMTT, dump page tabel and context switch Trace\n");
                return -8;
        }
	if(argc == 3)
        {
                is_pt_addr_trace = atoi(argv[2]);
        }

	 /**********************************************************************************************************/
        if( cmd != 2 )
        {
        // we want to start HMTT Trace
                hmtt_control_fd = open(hmtt_control_path, O_RDWR);
                if( hmtt_control_fd < 0 )
                {
                        printf("#### Failed to open %s, errno = %d\n", hmtt_control_path, errno);
                        return -8;
                }
                printf("$$$$ Open dev %s successfully\n", hmtt_control_path);
                config_space_base = mmap( 0,CFG_SIZE,PROT_READ|PROT_WRITE,MAP_SHARED,hmtt_control_fd,0 );
                if( config_space_base < 0 ){
                        printf("#### Failed to mmap %s into process space, errno=%d\n", hmtt_control_path, errno);
                        return -1;
                }
                printf("$$$$ mmap %s into process addr space successfully\n", hmtt_control_path);

                //send a tag to hmtt for start trace
                ch = *(char *)(config_space_base + 64*CMD_STOP_TRACE);
                ch = *(char *)(config_space_base + 64*CMD_STOP_TRACE);
                ch = *(char *)(config_space_base + 64*CMD_STOP_TRACE);
                ch = *(char *)(config_space_base + 64*CMD_STOP_TRACE);
                ch = *(char *)(config_space_base + 64*CMD_STOP_TRACE);
                ch = *(char *)(config_space_base + 64*CMD_STOP_TRACE);
        }
        /*********************************************************************************************************/
   
#if 1


    //start trace
    if(cmd == 0)//sata_dma
	//do nothing
	;
    else if(cmd == 1)//e1k_dma
    	stop_page_table_trace_test(fd);
    else if(cmd == 2)//ide and e1k
        stop_page_table_trace_test(fd);
    //else if(cmd == 3)//ide and e1k
    //    stop_page_table_trace_test(fd);
    //else if(cmd == 4)//ide and e1k
    //    stop_dump_page_table_trace_test(fd);
    //else if(cmd == 5)//ide and e1k
    //    stop_pid_comm_trace_test(fd);
    //else if(cmd == 6)//ide and e1k
    //    stop_context_switch_trace_test(fd);
    /*else if(cmd == 99)//ide and e1k
        stop_all_trace_test(fd);
    else if(cmd == 100)//ide and e1k
        stop_hmtt_trace(fd);*/
    else
       printf("param error!\n");

#endif
 	if( cmd != 2 )
        {
                munmap( config_space_base, CFG_SIZE );
                close(hmtt_control_fd);
        }

    	close(fd);

	printf("$$$$ Exit from stop_hmtt_trace\n");

    return 0;
}
