/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "COPYING_Apache"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */


#ifndef _KERNEL_TRACE_HMTT3_
#define _KERNEL_TRACE_HMTT3_


/*********************************************************************************************
** this is used to control the HMTT3 card to start, stop collecting memory trace, 
** just ignore it or modify it to your self control configuration 
**  
**
**********************************************************************************************/
//the value should be the same as that of "../cfg_content"

#define cfg_start             0x0000000130000000ULL       //3.75GB  
#define cfg_end               0x0000000130001000ULL       // +4KB,at least one page size

#define CMD_IDLE                       0
#define CMD_START_TRACE                1        //start_trace: start HMTT3 to collect memory trace 
#define CMD_STOP_TRACE                 2        //stop_trace: stop HMTT3 from collecting memory trace
#define CMD_PAUSE_TRACE                3
#define CMD_SOFT_RESET                 4        // soft_reset:  reset the user logic
#define CMD_HARD_RESET                 5        // hard_reset:  reset the user logic and the pcie core

/*********************************************************************************************/

#define KERNEL_TRACE_ENTRY_ADDR (4928ULL)   //in MB, 3.875GB
#define KERNEL_TRACE_SIZE (65LLU)           //in total 66MB, the first 1MB is used for config cmd, the next 65MB is used for kernel trace buf
#define KERNEL_TRACE_WRRD_PTR_ENTRY_BASE KERNEL_TRACE_ENTRY_ADDR
#define KERNEL_TRACE_WRRD_PTR_SIZE (1LLU)
#define KERNEL_TRACE_BUF_ENTRY_ADDR (KERNEL_TRACE_ENTRY_ADDR + KERNEL_TRACE_WRRD_PTR_SIZE)
#define KERNEL_TRACE_BUF_SIZE (64LLU)




#define KERNEL_TRACE_CONFIG_ENTRY_ADDR (5056ULL)     //4616, 2568
#define KERNEL_TRACE_CONFIG_SIZE (1LLU) //in MB, we just want to dump kernel trace, and not config tag mem access here
#define KERNEL_TRACE_SEQ_NUM     (64LLU)



//~for kernel trace event
//#define USE_IDE
//#define USE_E1K
//#define USE_SATA
#define USE_PAGE_TABLE
//#define USE_PID_COMM
#define USE_CONTEXT_SWITCH
#define USE_PT_ADDR

//~for kernel trace control command
#define CMD_END_AND_DUMP_TRACE             0
#define CMD_IDE_START_TRACE                1
#define CMD_IDE_STOP_TRACE                 2
#define CMD_IDE_STOP_AND_CLEAR_TRACE       3
#define CMD_E1K_START_TX_TRACE             4
#define CMD_E1K_STOP_TX_TRACE              5
#define CMD_E1K_START_RX_TRACE             6
#define CMD_E1K_STOP_RX_TRACE              7
#define CMD_E1K_STOP_AND_CLEAR_TX_TRACE    8
#define CMD_E1K_STOP_AND_CLEAR_RX_TRACE    9
#define CMD_IOREMAP_WRITE_TEST             10
#define CMD_SATA_START_TRACE               11
#define CMD_SATA_STOP_TRACE                12
#define CMD_SATA_STOP_AND_CLEAR_TRACE      13
#define CMD_DUMP_PAGE_TABLE_TRACE          14
#define CMD_PAGE_TABLE_START_TRACE         15
#define CMD_PAGE_TABLE_STOP_TRACE          16
#define CMD_PAGE_TABLE_STOP_AND_CLEAR_TRACE   17
#define CMD_PID_COMM_START_TRACE           18
#define CMD_PID_COMM_STOP_TRACE            19
#define CMD_PID_COMM_STOP_AND_CLEAR_TRACE  20
#define CMD_CONTEXT_SWITCH_START_TRACE           21
#define CMD_CONTEXT_SWITCH_STOP_TRACE            22
#define CMD_CONTEXT_SWITCH_STOP_AND_CLEAR_TRACE  23

#define CMD_BEGIN_KERNEL_TRACE 24
#define CMD_END_KERNEL_TRACE 25
#define CMD_WRITE_KERNEL_TRACE_END_TAG 26

#define CMD_PT_ADDR_START_TRACE			27
#define CMD_PT_ADDR_STOP_TRACE			28

//#define CMD_START_TRACE                    50
//#define CMD_STOP_TRACE                     51
#define CMD_RESET_BUFFER                   52


//~for kernel_trace sync tag uncachable tag memory access
#define SET_PTE_TAG 			0
#define FREE_PTE_TAG 			1
//#define DMA_BEGIN_TAG         		2
//#define DMA_END_TAG           		3
//#define DMA_DESC_TBL_TAG      		4
//#define DMA_ISSUECMD_TAG      		5
//#define SEMANTIC_TAG_END      		6

//#define SCHED_TAG_BASE        		7
//#define SCHED_TAG_REGION      		8
//#define SCHED_PREV_PID_TAG   		9
//#define SCHED_NEXT_PID_TAG   		10

#define DUMP_PAGE_TABLE_TAG             2
#define KERNEL_TRACE_END_TAG            3
#define CONTEXT_SWITCH_TAG              4

#define SET_PT_ADDR_TAG                 5
#define FREE_PT_ADDR_TAG                6

#define TAG_ACCESS_SIZE (4096LLU)
#define TAG_ACCESS_STEP (2048LLU)   //in Byte, we access 2 addr with step 2048B to confirm that the kernel event has happened
#define TAG_ACCESS_TIMES (2U)
#define TAG_MAX_POS (256U)

#endif
